import java.awt.*;
import java.awt.image.*;

public class SphereTexture extends Frame{
	static int WIDTH = 512;
	static int HEIGHT = 512;
	int coords[][] = new int[WIDTH][HEIGHT];
	boolean dirty = true;
	Image offImage;
	int count = 1000;
	
	public SphereTexture(){
		super();
		setBackground( Color.red );
	}
	
	void sliceIterate(){
		double x, y, z;
		double nx = Math.random()*2-1;
		double ny = Math.random()*2-1;
		double nz = Math.random()*2-1;
		double factor = Math.random()/Math.sqrt(nx*nx+ny*ny+nz*nz);
		
		nx /= factor;
		ny /= factor;
		nz /= factor;
		
		for( int h = 0; h < WIDTH; h++ ){
			for( int v = 0; v < HEIGHT; v++ ){
				double theta = h*2*Math.PI/WIDTH;
				double phi = v*Math.PI/HEIGHT - Math.PI/2;

				x = Math.cos(theta)*Math.cos(phi) - nx;
				y = Math.sin(theta)*Math.cos(phi) - ny;
				z = Math.sin(phi) - nz;
				
				if( x*nx + y*ny + z*nz > 0 )
					coords[h][v]++;
			}
		}
	}

	public void paint( Graphics g ){
		if( offImage != null && !dirty ){
			g.drawImage( offImage, 15, 35, null );
			return;
		}
		int pix[] = new int[WIDTH*HEIGHT];
		int index = 0;
		float min = count;
		float max = 0;
		for( int v = 0; v < HEIGHT; v++ ){
			for( int h = 0; h < WIDTH; h++ ){
				if( coords[h][v] < min )
					min = coords[h][v];
				if( coords[h][v] > max )
					max = coords[h][v];
			}
		}
		for( int v = 0; v < HEIGHT; v++ ){
			for( int h = 0; h < WIDTH; h++ ){
				pix[index++] = Color.getHSBColor( 0, 0, (coords[h][v]-min)/(float)(max-min) ).getRGB();
			}
		}
		offImage = createImage( new MemoryImageSource( WIDTH, HEIGHT, pix, 0, WIDTH ) );
		g.drawImage( offImage, 15, 35, this );
		dirty = false;
	}
	
	public static void main( String[] args ){
		SphereTexture s = new SphereTexture();
		if( args.length > 0 )
			s.count = Integer.parseInt( args[0] );
		if( args.length > 2 ){
			SphereTexture.WIDTH = Integer.parseInt( args[1] );
			SphereTexture.HEIGHT = Integer.parseInt( args[2] );
		}
		s.setBounds( 50, 50, WIDTH + 50, HEIGHT + 50 );
		s.setVisible( true );
		for( int i = 0; i < s.count; i++ ){
			s.sliceIterate();
			System.out.print( "." );
			if( i % 25 == 24 ){
				s.dirty = true;
				s.repaint();
			}
			if( i % (s.count/10) == s.count/10-1 )
				System.out.println( "" + (i+1)/(float)s.count*100 + "%" );
		}
	}
}